@@ -157,7 +157,7 @@
 # include <drm/drm_device.h>
 # include <drm/drm_ioctl.h>
 # include <drm/drm_fourcc.h>
-# if RTLNX_VER_MAX(5,15,0)
+# if RTLNX_VER_MAX(5,15,0) && !RTLNX_RHEL_MAJ_PREREQ(9,1)
 #  include <drm/drm_irq.h>
 # endif
 # include <drm/drm_vblank.h>
@@ -175,7 +175,7 @@
 #include <drm/ttm/ttm_bo_api.h>
 #include <drm/ttm/ttm_bo_driver.h>
 #include <drm/ttm/ttm_placement.h>
-#if RTLNX_VER_MAX(5,13,0)
+#if RTLNX_VER_MAX(5,13,0) && !RTLNX_RHEL_RANGE(8,6, 8,99)
 # include <drm/ttm/ttm_memory.h>
 #endif
 #if RTLNX_VER_MAX(5,12,0) && !RTLNX_RHEL_MAJ_PREREQ(8,5)
@@ -185,6 +185,10 @@
 # include <drm/ttm/ttm_resource.h>
 #endif
 
+#if RTLNX_VER_MIN(6,0,0)
+# include <drm/drm_framebuffer.h>
+#endif
+
 #include "vboxvideo_guest.h"
 #include "vboxvideo_vbe.h"
 #include "hgsmi_ch_setup.h"
@@ -232,7 +236,7 @@ static inline void drm_gem_object_put(struct drm_gem_object *obj)
 /** Field "pdev" of struct drm_device was removed in 5.14. This macro
  * transparently handles this change. Input argument is a pointer
  * to struct drm_device. */
-#if RTLNX_VER_MIN(5,14,0)
+#if RTLNX_VER_MIN(5,14,0) || RTLNX_RHEL_RANGE(8,6, 8,99)
 # define VBOX_DRM_TO_PCI_DEV(_dev) to_pci_dev(_dev->dev)
 #else
 # define VBOX_DRM_TO_PCI_DEV(_dev) _dev->pdev
@@ -276,7 +280,7 @@ struct vbox_private {
 		struct drm_global_reference mem_global_ref;
 		struct ttm_bo_global_ref bo_global_ref;
 #endif
-#if RTLNX_VER_MIN(5,13,0)
+#if RTLNX_VER_MIN(5,13,0) || RTLNX_RHEL_RANGE(8,6, 8,99)
 		struct ttm_device bdev;
 #else
 		struct ttm_bo_device bdev;
@@ -519,7 +523,7 @@ int vbox_gem_prime_mmap(struct drm_gem_object *obj,
 int vbox_irq_init(struct vbox_private *vbox);
 void vbox_irq_fini(struct vbox_private *vbox);
 void vbox_report_hotplug(struct vbox_private *vbox);
-#if RTLNX_VER_MAX(5,15,0)
+#if RTLNX_VER_MAX(5,15,0) && !RTLNX_RHEL_MAJ_PREREQ(9,1)
 irqreturn_t vbox_irq_handler(int irq, void *arg);
 #endif
 
