@@ -38,11 +38,6 @@
  * of the LGPL is applied is otherwise unspecified.
  */
 
-// Use VBE new dynamic mode list.  Note that without this option, no
-// checks are currently done to make sure that modes fit into the
-// framebuffer!
-#define VBE_NEW_DYN_LIST
-
 #include <inttypes.h>
 #include <stddef.h>
 #include "vbe.h"
@@ -53,7 +48,7 @@
 //#define VBE2_NO_VESA_CHECK
 
 // use bytewise i/o (Longhorn beta issue, not in released Vista)
-//#define VBE_BYTEWISE_IO
+#define VBE_BYTEWISE_IO
 
 #ifdef VBE_BYTEWISE_IO
     extern void do_out_dx_ax();
@@ -120,6 +115,12 @@ static void dispi_set_yres(uint16_t yres)
     out_w(VBE_DISPI_IOPORT_DATA, yres);
 }
 
+static uint16_t dispi_get_yres(void)
+{
+    out_w(VBE_DISPI_IOPORT_INDEX, VBE_DISPI_INDEX_YRES);
+    return in_w(VBE_DISPI_IOPORT_DATA);
+}
+
 static void dispi_set_bpp(uint16_t bpp)
 {
 #ifdef VGA_DEBUG
@@ -129,19 +130,44 @@ static void dispi_set_bpp(uint16_t bpp)
     out_w(VBE_DISPI_IOPORT_DATA, bpp);
 }
 
+static uint16_t dispi_get_bpp(void)
+{
+    out_w(VBE_DISPI_IOPORT_INDEX, VBE_DISPI_INDEX_BPP);
+    return in_w(VBE_DISPI_IOPORT_DATA);
+}
+
+static void dispi_set_virt_width(uint16_t vwidth)
+{
+#ifdef VGA_DEBUG
+    printf("vbe_set_virt_width: %04x\n", vwidth);
+#endif
+    out_w(VBE_DISPI_IOPORT_INDEX, VBE_DISPI_INDEX_VIRT_WIDTH);
+    out_w(VBE_DISPI_IOPORT_DATA, vwidth);
+}
+
+static uint16_t dispi_get_virt_width(void)
+{
+    out_w(VBE_DISPI_IOPORT_INDEX, VBE_DISPI_INDEX_VIRT_WIDTH);
+    return in_w(VBE_DISPI_IOPORT_DATA);
+}
+
+static uint16_t dispi_get_virt_height(void)
+{
+    out_w(VBE_DISPI_IOPORT_INDEX, VBE_DISPI_INDEX_VIRT_HEIGHT);
+    return in_w(VBE_DISPI_IOPORT_DATA);
+}
+
 uint16_t in_word(uint16_t port, uint16_t addr)
 {
     outw(port, addr);
     return inw(port);
 }
 
-#ifdef VBE_NEW_DYN_LIST
 uint8_t in_byte(uint16_t port, uint16_t addr)
 {
     outw(port, addr);
     return inb(port);
 }
-#endif
 
 /* Display "chip" identification helpers. */
 static uint16_t dispi_get_id(void)
@@ -173,12 +199,11 @@ void vbe_init(void)
 }
 
 /* Find the offset of the desired mode, given its number. */
-#ifdef VBE_NEW_DYN_LIST
 static uint16_t mode_info_find_mode(uint16_t mode, Boolean using_lfb)
 {
     uint16_t    sig, vmode, attrs;
     uint16_t    cur_info_ofs;   /* Current offset in mode list. */
-    
+
     /* Read and check the VBE Extra Data signature. */
     sig = in_word(VBE_EXTRA_PORT, 0);
     if (sig != VBEHEADER_MAGIC) {
@@ -187,14 +212,14 @@ static uint16_t mode_info_find_mode(uint16_t mode, Boolean using_lfb)
 #endif
         return 0;
     }
-    
+
     cur_info_ofs = sizeof(VBEHeader);
-    
+
     vmode = in_word(VBE_EXTRA_PORT, cur_info_ofs + offsetof(ModeInfoListItem, mode)/*&cur_info->mode*/);
     while (vmode != VBE_VESA_MODE_END_OF_LIST)
     {
         attrs = in_word(VBE_EXTRA_PORT, /*&cur_info->info.ModeAttributes*/cur_info_ofs + offsetof(ModeInfoListItem, info.ModeAttributes) );
-        
+
         if (vmode == mode)
         {
             if (!using_lfb)
@@ -212,27 +237,6 @@ static uint16_t mode_info_find_mode(uint16_t mode, Boolean using_lfb)
     }
     return 0;
 }
-#else
-static ModeInfoListItem* mode_info_find_mode(uint16_t mode, Boolean using_lfb)
-{
-    ModeInfoListItem    *cur_info = &mode_info_list;
-    
-    while (cur_info->mode != VBE_VESA_MODE_END_OF_LIST)
-    {
-        if (cur_info->mode == mode)
-        {
-            if (!using_lfb)
-                return cur_info;
-            else if (cur_info->info.ModeAttributes & VBE_MODE_ATTRIBUTE_LINEAR_FRAME_BUFFER_MODE)
-                return cur_info;
-            else
-                cur_info++;
-        } else
-            cur_info++;
-    }
-    return 0;
-}
-#endif
 
 #ifndef VBOX
 ; VBE Display Info - Display information on screen about the VBE
@@ -269,18 +273,13 @@ void vbe_biosfn_return_controller_information(uint16_t STACK_BASED *AX, uint16_t
     uint16_t            vbe2_info;
     uint16_t            cur_mode = 0;
     uint16_t            cur_ptr=34;
-#ifdef VBE_NEW_DYN_LIST
     uint16_t            cur_info_ofs;
     uint16_t            sig, vmode;
-#else
-    ModeInfoListItem    *cur_info = &mode_info_list;
-#endif
     uint16_t            max_bpp = dispi_get_max_bpp();
     VbeInfoBlock __far  *info_block;
 
     info_block = ES :> (VbeInfoBlock *)DI;
 
-#ifdef VBE_NEW_DYN_LIST
     /* Read VBE Extra Data signature */
     sig = in_word(VBE_EXTRA_PORT, 0);
     if (sig != VBEHEADER_MAGIC)
@@ -292,7 +291,6 @@ void vbe_biosfn_return_controller_information(uint16_t STACK_BASED *AX, uint16_t
         return;
     }
     cur_info_ofs = sizeof(VBEHeader);
-#endif
     status = *AX;
 
 #ifdef VGA_DEBUG
@@ -350,7 +348,6 @@ void vbe_biosfn_return_controller_information(uint16_t STACK_BASED *AX, uint16_t
         info_block->OemProductRev.Ptr  = &vbebios_product_revision;
     }
 
-#ifdef VBE_NEW_DYN_LIST
     do
     {
         uint8_t     data_b;
@@ -372,23 +369,6 @@ void vbe_biosfn_return_controller_information(uint16_t STACK_BASED *AX, uint16_t
 
     // Add vesa mode list terminator
     write_word(ES, DI + cur_ptr, vmode);
-#else
-    do
-    {
-        if (cur_info->info.BitsPerPixel <= max_bpp) {
-#ifdef VGA_DEBUG
-            printf("VBE found mode %x => %x\n", cur_info->mode,cur_mode);
-#endif
-            write_word(ES, DI + cur_ptr, cur_info->mode);
-            cur_mode++;
-            cur_ptr += 2;
-        }
-        cur_info++;
-    } while (cur_info->mode != VBE_VESA_MODE_END_OF_LIST);
-
-    // Add vesa mode list terminator
-    write_word(ES, DI + cur_ptr, cur_info->mode);
-#endif // VBE_NEW_DYN_LIST
     *AX = 0x004F;
 }
 
@@ -405,11 +385,7 @@ void vbe_biosfn_return_controller_information(uint16_t STACK_BASED *AX, uint16_t
 void vbe_biosfn_return_mode_information(uint16_t STACK_BASED *AX, uint16_t CX, uint16_t ES, uint16_t DI)
 {
     uint16_t            result = 0x0100;
-#ifdef VBE_NEW_DYN_LIST
     uint16_t            cur_info_ofs;
-#else
-    ModeInfoListItem    *cur_info;
-#endif
     Boolean             using_lfb;
     uint8_t             win_attr;
 
@@ -420,30 +396,20 @@ void vbe_biosfn_return_mode_information(uint16_t STACK_BASED *AX, uint16_t CX, u
     using_lfb = ((CX & VBE_MODE_LINEAR_FRAME_BUFFER) == VBE_MODE_LINEAR_FRAME_BUFFER);
     CX = (CX & 0x1ff);
 
-#ifdef VBE_NEW_DYN_LIST
     cur_info_ofs = mode_info_find_mode(CX, using_lfb);
 
     if (cur_info_ofs) {
         uint16_t    i;
-#else
-    cur_info = mode_info_find_mode(CX, using_lfb);
-
-    if (cur_info != 0) {
-#endif
 #ifdef VGA_DEBUG
         printf("VBE found mode %x\n",CX);
 #endif
         memsetb(ES, DI, 0, 256);    // The mode info size is fixed
-#ifdef VBE_NEW_DYN_LIST
         for (i = 0; i < sizeof(ModeInfoBlockCompact); i++) {
             uint8_t b;
 
             b = in_byte(VBE_EXTRA_PORT, cur_info_ofs + offsetof(ModeInfoListItem, info) + i/*(char *)(&(cur_info->info)) + i*/);
             write_byte(ES, DI + i, b);
         }
-#else
-        memcpyb(ES, DI, 0xc000, &(cur_info->info), sizeof(ModeInfoBlockCompact));
-#endif
         win_attr = read_byte(ES, DI + offsetof(ModeInfoBlock, WinAAttributes));
         if (win_attr & VBE_WINDOW_ATTRIBUTE_RELOCATABLE) {
             write_word(ES, DI + offsetof(ModeInfoBlock, WinFuncPtr), (uint16_t)(dispi_set_bank_farcall));
@@ -478,11 +444,7 @@ void vbe_biosfn_return_mode_information(uint16_t STACK_BASED *AX, uint16_t CX, u
 void vbe_biosfn_set_mode(uint16_t STACK_BASED *AX, uint16_t BX, uint16_t ES, uint16_t DI)
 {
     uint16_t            result;
-#ifdef VBE_NEW_DYN_LIST
     uint16_t            cur_info_ofs;
-#else
-    ModeInfoListItem    *cur_info;
-#endif
     Boolean             using_lfb;
     uint8_t             no_clear;
     uint8_t             lfb_flag;
@@ -507,7 +469,6 @@ void vbe_biosfn_set_mode(uint16_t STACK_BASED *AX, uint16_t BX, uint16_t ES, uin
         goto leave;
     }
 
-#ifdef VBE_NEW_DYN_LIST
     cur_info_ofs = mode_info_find_mode(BX, using_lfb);
 
     if (cur_info_ofs != 0)
@@ -523,41 +484,18 @@ void vbe_biosfn_set_mode(uint16_t STACK_BASED *AX, uint16_t BX, uint16_t ES, uin
         printf("VBE found mode %x, setting:\n", BX);
         printf("\txres%x yres%x bpp%x\n", xres, yres, bpp);
 #endif
-#else
-    cur_info = mode_info_find_mode(BX, using_lfb);
-
-    if (cur_info != 0)
-    {
-#ifdef VGA_DEBUG
-        printf("VBE found mode %x, setting:\n", BX);
-        printf("\txres%x yres%x bpp%x\n",
-                cur_info->info.XResolution,
-                cur_info->info.YResolution,
-                cur_info->info.BitsPerPixel);
-#endif
-#endif // VBE_NEW_DYN_LIST
 
         // first disable current mode (when switching between vesa modi)
         dispi_set_enable(VBE_DISPI_DISABLED);
 
-#ifdef VBE_NEW_DYN_LIST
         if (bpp == 4)
-#else
-        if (cur_info->info.BitsPerPixel == 4)
-#endif
         {
             biosfn_set_video_mode(0x6a);
         }
 
-#ifdef VBE_NEW_DYN_LIST
         dispi_set_bpp(bpp);
         dispi_set_xres(xres);
         dispi_set_yres(yres);
-#else
-        dispi_set_bpp(cur_info->info.BitsPerPixel);
-        dispi_set_xres(cur_info->info.XResolution);
-        dispi_set_yres(cur_info->info.YResolution);
-#endif
         dispi_set_bank(0);
         dispi_set_enable(VBE_DISPI_ENABLED | no_clear | lfb_flag);
         vga_compat_setup();
@@ -649,7 +587,7 @@ void vbe_biosfn_restore_video_state(uint16_t ES, uint16_t BX)
  *              BX      = Number of 64-byte blocks to hold the state buffer (if DL=00h)
  *
  */
-void vbe_biosfn_save_restore_state(uint16_t STACK_BASED *AX, uint16_t CX, uint16_t DX, 
+void vbe_biosfn_save_restore_state(uint16_t STACK_BASED *AX, uint16_t CX, uint16_t DX,
                                    uint16_t ES, uint16_t STACK_BASED *BX)
 {
     uint16_t    result, val;
@@ -690,3 +628,175 @@ void vbe_biosfn_save_restore_state(uint16_t STACK_BASED *AX, uint16_t CX, uint16
     }
     *AX = result;
 }
+
+/** Function 06h - Set/Get Logical Scan Line Length
+ *
+ *  Input:
+ *              AX      = 4F06h
+ *              BL      = 00h Set Scan Line Length in Pixels
+ *                      = 01h Get Scan Line Length
+ *                      = 02h Set Scan Line Length in Bytes
+ *                      = 03h Get Maximum Scan Line Length
+ *              CX      = If BL=00h Desired Width in Pixels
+ *                        If BL=02h Desired Width in Bytes
+ *                        (Ignored for Get Functions)
+ *
+ *  Output:
+ *              AX      = VBE Return Status
+ *              BX      = Bytes Per Scan Line
+ *              CX      = Actual Pixels Per Scan Line (truncated to
+ *                        nearest complete pixel)
+ *              DX      = Maximum Number of Scan Lines
+ */
+void vbe_biosfn_get_set_scanline_length(uint16_t STACK_BASED *AX, uint16_t STACK_BASED *BX,
+                                        uint16_t STACK_BASED *CX, uint16_t STACK_BASED *DX)
+{
+    uint16_t    val;
+    uint16_t    result;
+    uint8_t     bpp;
+    uint8_t     subfn;
+    uint16_t    old_vw;
+
+    bpp    = dispi_get_bpp();
+    bpp    = bpp == 15 ? 16 : bpp;
+    old_vw = dispi_get_virt_width();
+    result = 0x004F;
+    val    = *CX;
+    subfn  = *BX & 0xFF;
+#ifdef VGA_DEBUG
+    printf("VBE get/set scanline len fn=%x, CX=%x\n", subfn, *CX);
+#endif
+    switch(subfn) {
+    case 0x02:
+        if (bpp == 4)
+            val = val * 8;
+        else
+            val = val / (bpp / 8);
+        /* fall through */
+    case 0x00:
+        dispi_set_virt_width(val);
+        /* fall through */
+    case 0x01:
+        val = dispi_get_virt_width();
+        *CX = val;                          /* Width in pixels. */
+        if (bpp == 4)
+            val = val / 8;
+        else
+            val = val * (bpp / 8);
+        val = (val + 3) & ~3;
+        *BX = val;                          /* Bytes per scanline. */
+        *DX = dispi_get_virt_height();      /* Height in lines. */
+        if (*DX < dispi_get_yres()) {
+            dispi_set_virt_width(old_vw);
+            result = 0x200;
+        }
+        break;
+    default:
+        // function failed
+        result = 0x100;
+        break;
+    }
+    *AX = result;
+}
+
+
+/* We would very much like to avoid dragging in the long multiply library
+ * routine, and we really just need to multiply two 16-bit numbers to
+ * obtain a 32-bit result, so...
+ */
+uint32_t mul32_16x16(uint16_t a, uint16_t b);
+#pragma aux mul32_16x16 =   \
+    "mul    dx"             \
+    parm [ax] [dx] modify nomemory;
+
+
+/** Private INT 10h function 5642h - Manage custom video modes using X/Y
+ *  resolution and bit depth rather than mode number
+ *
+ *  Input:
+ *              AX      = 5642h ('VB')
+ *              BL      = 00h Set video mode
+ *              BH      = If BL=00h Desired bit depth in pixels
+ *              CX      = If BL=00h Desired width in pixels
+ *              DX      = If BL=00h Desired height in pixels
+ *
+ *  Output:
+ *              AX      = VBE style return status
+ */
+void private_biosfn_custom_mode(uint16_t STACK_BASED *AX, uint16_t STACK_BASED *BX,
+                                uint16_t STACK_BASED *CX, uint16_t STACK_BASED *DX)
+{
+    uint16_t    result;
+    uint8_t     subfn;
+    uint8_t     bpp;
+    uint8_t     lfb_flag;
+    uint16_t    xres;
+    uint16_t    yres;
+    uint16_t    line_size;
+    uint32_t    vram_size;
+    uint32_t    mode_size;
+
+    result = 0x004F;
+    subfn  = *BX & 0xFF;
+    switch (subfn) {
+    case 0x00:
+        xres = *CX;
+        yres = *DX;
+        bpp  = (*BX >> 8) & 0x7F;
+#ifdef VGA_DEBUG
+        printf("Set custom mode %04x by %04x %xbpp\n", xres, yres, bpp);
+#endif
+        /* Only allow 32/16/8bpp. */
+        if (bpp != 8 && bpp != 16 && bpp != 32) {
+            result = 0x100;
+            break;
+        }
+
+        /* Determine the LFB flag. */
+        lfb_flag = *BX & 0x8000 ? VBE_DISPI_LFB_ENABLED : 0;
+
+        /* Cap the resolution to something not insanely high or low. */
+        if (xres < 640)
+            xres = 640;
+        else if (xres > 2560)
+            xres = 2560;
+        if (yres < 480)
+            yres = 480;
+        else if (yres > 1920)
+            yres = 1920;
+#ifdef VGA_DEBUG
+        printf("Adjusted resolution %04x by %04x\n", xres, yres);
+#endif
+
+        /* Calculate the VRAM size in bytes. */
+        vram_size = (uint32_t)in_word(VBE_EXTRA_PORT, 0xffff) << 16;
+
+        /* Calculate the scanline size in bytes. */
+        line_size = xres * (bpp / 8);
+        line_size = (line_size + 3) & ~3;
+        /* And now the memory required for the mode. */
+        mode_size = mul32_16x16(line_size, yres);
+
+        if (mode_size > vram_size) {
+            /* No can do. Don't have that much VRAM. */
+            result = 0x200;
+            break;
+        }
+
+        /* Mode looks valid, let's get cracking. */
+        dispi_set_enable(VBE_DISPI_DISABLED);
+        dispi_set_bpp(bpp);
+        dispi_set_xres(xres);
+        dispi_set_yres(yres);
+        dispi_set_bank(0);
+        dispi_set_enable(VBE_DISPI_ENABLED | lfb_flag);
+        vga_compat_setup();
+        break;
+
+    default:
+        // unsupported sub-function
+        result = 0x100;
+        break;
+    }
+    *AX = result;
+}
