@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2017 Oracle Corporation
+ * Copyright (C) 2006-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -215,23 +215,24 @@ static void vusbMsgCompletion(PVUSBURB pUrb)
  *
  * @returns true if it could be retried.
  * @returns false if it should be completed with failure.
+ * @param   pRh     The roothub the URB originated from.
  * @param   pUrb    The URB in question.
  */
-int vusbUrbErrorRh(PVUSBURB pUrb)
+int vusbUrbErrorRhEx(PVUSBROOTHUB pRh, PVUSBURB pUrb)
 {
     PVUSBDEV pDev = pUrb->pVUsb->pDev;
-    PVUSBROOTHUB pRh = vusbDevGetRh(pDev);
-    AssertPtrReturn(pRh, VERR_VUSB_DEVICE_NOT_ATTACHED);
     LogFlow(("%s: vusbUrbErrorRh: pDev=%p[%s] rh=%p\n", pUrb->pszDesc, pDev, pDev->pUsbIns ? pDev->pUsbIns->pszName : "", pRh));
+    RT_NOREF(pDev);
     return pRh->pIRhPort->pfnXferError(pRh->pIRhPort, pUrb);
 }
 
 /**
  * Does URB completion on roothub level.
  *
+ * @param   pRh     The roothub the URB originated from.
  * @param   pUrb    The URB to complete.
  */
-void vusbUrbCompletionRh(PVUSBURB pUrb)
+void vusbUrbCompletionRhEx(PVUSBROOTHUB pRh, PVUSBURB pUrb)
 {
     LogFlow(("%s: vusbUrbCompletionRh: type=%s status=%s\n",
              pUrb->pszDesc, vusbUrbTypeName(pUrb->enmType), vusbUrbStatusName(pUrb->enmStatus)));
@@ -249,9 +250,6 @@ void vusbUrbCompletionRh(PVUSBURB pUrb)
             LogRel(("VUSB: Capturing URB completion event failed with %Rrc\n", rc));
     }
 
-    PVUSBROOTHUB pRh = vusbDevGetRh(pUrb->pVUsb->pDev);
-    AssertPtrReturnVoid(pRh);
-
     /* If there is a sniffer on the roothub record the completed URB there too. */
     if (pRh->hSniffer != VUSBSNIFFER_NIL)
     {
@@ -303,10 +301,12 @@ void vusbUrbCompletionRh(PVUSBURB pUrb)
                     {
                         case VUSBSTATUS_OK:
                             if (cb)                     STAM_COUNTER_INC(&pRh->aStatIsocDetails[i].Ok);
-                            else                        STAM_COUNTER_INC(&pRh->aStatIsocDetails[i].Ok0); break;
+                            else                        STAM_COUNTER_INC(&pRh->aStatIsocDetails[i].Ok0);
+                            break;
                         case VUSBSTATUS_DATA_UNDERRUN:
                             if (cb)                     STAM_COUNTER_INC(&pRh->aStatIsocDetails[i].DataUnderrun);
-                            else                        STAM_COUNTER_INC(&pRh->aStatIsocDetails[i].DataUnderrun0); break;
+                            else                        STAM_COUNTER_INC(&pRh->aStatIsocDetails[i].DataUnderrun0);
+                            break;
                         case VUSBSTATUS_DATA_OVERRUN:   STAM_COUNTER_INC(&pRh->aStatIsocDetails[i].DataOverrun); break;
                         case VUSBSTATUS_NOT_ACCESSED:   STAM_COUNTER_INC(&pRh->aStatIsocDetails[i].NotAccessed); break;
                         default:                        STAM_COUNTER_INC(&pRh->aStatIsocDetails[i].Misc); break;
@@ -362,7 +362,7 @@ void vusbUrbCompletionRh(PVUSBURB pUrb)
 #endif
         case VUSBXFERTYPE_BULK:
             if (pUrb->enmStatus != VUSBSTATUS_OK)
-                vusbUrbErrorRh(pUrb);
+                vusbUrbErrorRhEx(pRh, pUrb);
             break;
     }
 #ifdef LOG_ENABLED
@@ -947,12 +947,14 @@ static int vusbUrbSubmitCtrl(PVUSBURB pUrb)
              * will be no status stage.
              */
             uint8_t *pbData = (uint8_t *)(pExtra->pMsg + 1);
-            if (&pExtra->pbCur[pUrb->cbData] > &pbData[pSetup->wLength])
+            if ((uintptr_t)&pExtra->pbCur[pUrb->cbData] > (uintptr_t)&pbData[pSetup->wLength])
             {
+                /** @todo r=bird: This code stinks, esp. the iPhone carp.  @bugref{9899} */
                 if (!pSetup->wLength) /* happens during iPhone detection with iTunes (correct?) */
                 {
-                    Log(("%s: vusbUrbSubmitCtrl: pSetup->wLength == 0!! (iPhone)\n", pUrb->pszDesc));
-                    pSetup->wLength = pUrb->cbData;
+                    Log(("%s: vusbUrbSubmitCtrl: pSetup->wLength == 0!! Changing it to RT_MIN(cbData=%u, cbMax=%u) (iPhone hack)\n",
+                         pUrb->pszDesc, pUrb->cbData, pExtra->cbMax));
+                    pSetup->wLength = RT_MIN(pUrb->cbData, pExtra->cbMax); /** @todo resize? */
                 }
 
                 /* Variable length data transfers */
@@ -960,10 +962,9 @@ static int vusbUrbSubmitCtrl(PVUSBURB pUrb)
                     ||  pSetup->wLength == 0
                     ||  (pUrb->cbData % pSetup->wLength) == 0)  /* magic which need explaining... */
                 {
-                    uint8_t *pbEnd = pbData + pSetup->wLength;
-                    int cbLeft = pbEnd - pExtra->pbCur;
+                    ssize_t const cbLeft = &pbData[pSetup->wLength] - pExtra->pbCur;
                     LogFlow(("%s: vusbUrbSubmitCtrl: Var DATA, pUrb->cbData %d -> %d\n", pUrb->pszDesc, pUrb->cbData, cbLeft));
-                    pUrb->cbData = cbLeft;
+                    pUrb->cbData = cbLeft >= 0 ? (uint32_t)cbLeft : 0;
                 }
                 else
                 {
@@ -995,6 +996,8 @@ static int vusbUrbSubmitCtrl(PVUSBURB pUrb)
             else
             {
                 /* get data for sending when completed. */
+                AssertStmt((ssize_t)pUrb->cbData <= pExtra->cbMax - (pExtra->pbCur - pbData), /* paranoia: checked above */
+                           pUrb->cbData = pExtra->cbMax - (uint32_t)RT_MIN(pExtra->pbCur - pbData, pExtra->cbMax));
                 memcpy(pExtra->pbCur, pUrb->abData, pUrb->cbData);
 
                 /* advance */
