@@ -5,7 +5,7 @@
  */
 
 /*
- * Copyright (C) 2010 Oracle Corporation
+ * Copyright (C) 2010-2020 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -23,7 +23,6 @@
 #include <VBox/com/string.h>
 #include <VBox/com/ErrorInfo.h>
 #include <VBox/com/errorprint.h>
-#include <VBox/com/EventQueue.h>
 
 #include <iprt/initterm.h>
 #include <iprt/stream.h>
@@ -85,7 +84,8 @@ void importOVF(const char *pcszPrefix,
                std::list<Guid> &llMachinesCreated)
 {
     char szAbsOVF[RTPATH_MAX];
-    RTPathAbs(pcszOVF0, szAbsOVF, sizeof(szAbsOVF));
+    RTPathExecDir(szAbsOVF, sizeof(szAbsOVF));
+    RTPathAppend(szAbsOVF, sizeof(szAbsOVF), pcszOVF0);
 
     RTPrintf("%s: reading appliance \"%s\"...\n", pcszPrefix, szAbsOVF);
     ComPtr<IAppliance> pAppl;
@@ -116,12 +116,12 @@ void importOVF(const char *pcszPrefix,
         com::SafeArray<VirtualSystemDescriptionType_T> aTypes;
         com::SafeArray<BSTR> aRefs;
         com::SafeArray<BSTR> aOvfValues;
-        com::SafeArray<BSTR> aVboxValues;
+        com::SafeArray<BSTR> aVBoxValues;
         com::SafeArray<BSTR> aExtraConfigValues;
         rc = pVSys->GetDescription(ComSafeArrayAsOutParam(aTypes),
                                    ComSafeArrayAsOutParam(aRefs),
                                    ComSafeArrayAsOutParam(aOvfValues),
-                                   ComSafeArrayAsOutParam(aVboxValues),
+                                   ComSafeArrayAsOutParam(aVBoxValues),
                                    ComSafeArrayAsOutParam(aExtraConfigValues));
         if (FAILED(rc)) throw MyError(rc, "VirtualSystemDescription::GetDescription() failed\n");
 
@@ -194,6 +194,10 @@ void importOVF(const char *pcszPrefix,
                     pcszType = "scsi";
                 break;
 
+                case VirtualSystemDescriptionType_HardDiskControllerVirtioSCSI:
+                    pcszType = "virtio-scsi";
+                break;
+
                 case VirtualSystemDescriptionType_HardDiskImage:
                     pcszType = "hd";
                 break;
@@ -226,7 +230,7 @@ void importOVF(const char *pcszPrefix,
             RTPrintf("  vsys %2u item %2u: type %2d (%s), ovf: \"%ls\", vbox: \"%ls\", extra: \"%ls\"\n",
                      u, u2, t, pcszType,
                      aOvfValues[u2],
-                     aVboxValues[u2],
+                     aVBoxValues[u2],
                      aExtraConfigValues[u2]);
         }
     }
@@ -266,11 +270,17 @@ void copyDummyDiskImage(const char *pcszPrefix,
                         std::list<Utf8Str> &llFiles2Delete,
                         const char *pcszDest)
 {
+    char szSrc[RTPATH_MAX];
+    char szDst[RTPATH_MAX];
+    RTPathExecDir(szSrc, sizeof(szSrc));
+    RTPathAppend(szSrc, sizeof(szSrc), "ovf-testcases/ovf-dummy.vmdk");
+    RTPathExecDir(szDst, sizeof(szDst));
+    RTPathAppend(szDst, sizeof(szDst), pcszDest);
     RTPrintf("%s: copying ovf-dummy.vmdk to \"%s\"...\n", pcszPrefix, pcszDest);
 
-    int vrc = RTFileCopy("ovf-testcases/ovf-dummy.vmdk", pcszDest);
+    int vrc = RTFileCopy(szSrc, szDst);
     if (RT_FAILURE(vrc)) throw MyError(0, Utf8StrFmt("Cannot copy ovf-dummy.vmdk to %s: %Rra\n", pcszDest, vrc).c_str());
-    llFiles2Delete.push_back(pcszDest);
+    llFiles2Delete.push_back(szDst);
 }
 
 /**
@@ -281,13 +291,15 @@ void copyDummyDiskImage(const char *pcszPrefix,
  */
 int main(int argc, char *argv[])
 {
-    RTR3Init();
+    RTR3InitExe(argc, &argv, 0);
 
+    RTEXITCODE rcExit = RTEXITCODE_SUCCESS;
     HRESULT rc = S_OK;
 
     std::list<Utf8Str> llFiles2Delete;
     std::list<Guid> llMachinesCreated;
 
+    ComPtr<IVirtualBoxClient> pVirtualBoxClient;
     ComPtr<IVirtualBox> pVirtualBox;
 
     try
@@ -299,16 +311,14 @@ int main(int argc, char *argv[])
         ComPtr<ISession> pSession;
 
         RTPrintf("Creating VirtualBox object...\n");
-        rc = pVirtualBox.createLocalObject(CLSID_VirtualBox);
+        rc = pVirtualBoxClient.createInprocObject(CLSID_VirtualBoxClient);
+        if (SUCCEEDED(rc))
+            rc = pVirtualBoxClient->COMGETTER(VirtualBox)(pVirtualBox.asOutParam());
         if (FAILED(rc)) throw MyError(rc, "failed to create the VirtualBox object!\n");
 
         rc = pSession.createInprocObject(CLSID_Session);
         if (FAILED(rc)) throw MyError(rc, "failed to create a session object!\n");
 
-        // create the event queue
-        // (here it is necessary only to process remaining XPCOM/IPC events after the session is closed)
-        EventQueue eventQ;
-
         // for each testcase, we will copy the dummy VMDK image to the subdirectory with the OVF testcase
         // so that the import will find the disks it expects; this is just for testing the import since
         // the imported machines will obviously not be usable.
@@ -333,6 +343,7 @@ int main(int argc, char *argv[])
     {
         rc = e.m_rc;
         RTPrintf("%s", e.m_str.c_str());
+        rcExit = RTEXITCODE_FAILURE;
     }
 
     try
@@ -355,7 +366,7 @@ int main(int argc, char *argv[])
             if (FAILED(rc)) throw MyError(rc, "Machine::Unregister() failed\n");
 
             ComPtr<IProgress> pProgress;
-            rc = pMachine->Delete(ComSafeArrayAsInParam(sfaMedia), pProgress.asOutParam());
+            rc = pMachine->DeleteConfig(ComSafeArrayAsInParam(sfaMedia), pProgress.asOutParam());
             if (FAILED(rc)) throw MyError(rc, "Machine::DeleteSettings() failed\n");
             rc = pProgress->WaitForCompletion(-1);
             if (FAILED(rc)) throw MyError(rc, "Progress::WaitForCompletion() failed\n");
@@ -365,6 +376,7 @@ int main(int argc, char *argv[])
     {
         rc = e.m_rc;
         RTPrintf("%s", e.m_str.c_str());
+        rcExit = RTEXITCODE_FAILURE;
     }
 
     // clean up the VMDK copies that we made in copyDummyDiskImage()
@@ -378,11 +390,12 @@ int main(int argc, char *argv[])
     }
 
     pVirtualBox.setNull();
+    pVirtualBoxClient.setNull();
 
     RTPrintf("Shutting down COM...\n");
     com::Shutdown();
-    RTPrintf ("tstOVF all done!\n");
+    RTPrintf("tstOVF all done: %s\n", rcExit ? "ERROR" : "SUCCESS");
 
-    return rc;
+    return rcExit;
 }
 
